/*--------------------------------------------------------------------
 * $Id: yang56.c,v 1.2 2001/12/17 09:05:17 pa2h Exp $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2001 by Arve Kylling and Bernhard Mayer.
 * 
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/


/* This module was implemented by Albano Gonzales, */
/* eMail aglezf@ull.es                             */

#include <stdio.h>
#include <string.h>
#include <math.h>

/*agf*/
#define MAXWL 80   /*Number of wavelengths in tables (max)*/
#define MAXHABIT 7 /*Number of habits*/

#define EXT56NF -2 /*File ext56.table not found*/
#define SSA56NF -3 /*File ext56.table not found*/
#define G56NF   -4 /*File ext56.table not found*/
#define WLOUT   -5 /*Wavelength out of range*/
#define HABITNF -6 /*Habit not found*/

/*agf*/


/*Global variables for 8-13um parametrization (only for hexagons)*/
static float wlp[12]={8.0,8.5,9.0,9.5,10.0,10.5,10.8,11.0,11.5,12.0,12.5,13.0};
static float qep[3][12]=
{{1.989,1.979,1.982,1.992,1.984,2.032,2.036,2.029,2.012,2.006,2.000,3.201},
 {4.336,6.117,5.219,3.891,5.208,-3.643,-6.777,-5.179,-1.431,1.283,2.787,3.201},
 {-27.76,-48.77,-42.48,-37.43,-81.94,-40.75,5.749,3.789,-10.61,-28.0,-33.23,-26.64}};
static float ssp[3][12]=
{{0.5294,0.5205,0.5198,0.5191,0.5116,0.523,0.5384,0.5475,0.5562,0.5633,0.5556,0.5678},
 {0.2536,1.421,1.178,1.04,1.77,-0.8619,-2.454,-2.552,-1.974,-1.919,-1.605,-2.051},
 {15.41,7.036,9.676,10.63,-1.811,1.271,12.25,14.17,9.011,9.232,7.246,12.37}};
static float gp[2][12]=
{{0.8317,0.8217,0.8399,0.856,0.8917,0.9271,0.9092,0.908,0.8922,0.8519,0.8553,0.829},
 {0.03279,0.03565,0.0312,0.02779,0.02092,0.01387,0.01771,0.01501,0.01394,0.02355,0.01932,0.02682}};



/*agf*/
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
Function yang56.
Computes the parametrization of single scattering properties
for 7 habits, for 0.2<wavelength<4.8 um.

Input:  
wavel -> wavelength
reff -> Effective radius (could be computed using de2re)
habit -> name of habit: Solid-Column 0, Hollow-Column 1, Rough-Aggregate 2,
 			Rosette-4 3, Rosette-6 4, Plate 5 or Dendrite?. 
			(case insensitive)

Output:
*exti -> pointer to interp. volume extinction coefficient per ice water content.
*ssai -> pointer to interp. single scattering albedo.
*gi -> pointer to interp. asymmetry parameter.
*g2i -> pointer to interp. 2nd asymmetry parameter (double Henyey-Greenstaein).
*fi -> pointer to interp. factor f in double Henyey-Greenstaein function.

++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


int yang56(float wavel, float reff, int nhabit, char *path, float *exti, 
           float *ssai, float *gi, float *g2i, float *fi)
{
  static short filere=0;  /*Only read data files once*/
  static short flagwl=0;
  static float ext[MAXHABIT][4][MAXWL];
  static float ssa[MAXHABIT][4][MAXWL];
  static float g[MAXHABIT][6][MAXWL];
  static float wl[MAXWL];   
  static char habit_class[MAXHABIT][80];
  static int index;
  static float wlmin, wlmax;
  
  char habit_name[80];
  FILE *fp;

  char filename[256];
 
  int i,j;
  float auxd, auxd1, auxd2;
  int ini,fin,med;

  /*Read the data files*/
  if(filere==0)
  {
    filere++;

    /*extinction coefficient*/
    sprintf(filename,"%sext56.table",path);
    fp=fopen(filename,"r");
    if(fp==0)
    {
      fprintf(stderr,"Error, %s not found\n", filename);
      return EXT56NF;
    }
    
    index=0;

    fscanf(fp,"%s",habit_class[0]);
    fseek(fp,0,0);
    do{
      fscanf(fp,"%s %f %f %f %f %f", habit_name, wl+index, 
             &ext[0][0][index], &ext[0][1][index], &ext[0][2][index], 
             &ext[0][3][index]);
      index++;

    }while(strcasecmp(habit_name,habit_class[0])==0);

    index--;

    ext[1][0][0] = ext[0][0][index]; 
    ext[1][1][0] = ext[0][1][index]; 
    ext[1][2][0] = ext[0][2][index]; 
    ext[1][3][0] = ext[0][3][index]; 

    for(j=1;j<MAXHABIT;j++)
      for(i=1;i<index;i++)
        fscanf(fp,"%s %f %f %f %f %f",habit_name,&auxd, 
               &ext[j][0][i], &ext[j][1][i], &ext[j][2][i], 
               &ext[j][3][i]);

    wlmin=wl[0];
    wlmax=wl[index-1];
   
    fclose(fp);

    /*------------------------------------*/
    /*single scattering albedo*/
    sprintf(filename,"%sssa56.table",path);
    fp=fopen(filename,"r");
    if(fp==0)
    {
      fprintf(stderr,"Error, %s not found\n", filename);
      return SSA56NF;
    }

    for(j=0;j<MAXHABIT;j++)
      for(i=0;i<index;i++)
        fscanf(fp,"%s %f %f %f %f %f",habit_name,&auxd, 
               &ssa[j][0][i], &ssa[j][1][i], &ssa[j][2][i], 
               &ssa[j][3][i]);

    fclose(fp);

    /*------------------------------------*/
    /*asymetry parameter*/
    sprintf(filename,"%sg56.table",path);
    fp=fopen(filename,"r");
    if(fp==0)
    {
      fprintf(stderr,"Error, %s not found\n", filename);
      return G56NF;
    }

    for(j=0;j<MAXHABIT;j++)
      for(i=0;i<index;i++)
        fscanf(fp,"%s %f %f %f %f %f %f %f",habit_name,&auxd, 
               &g[j][0][i], &g[j][1][i], &g[j][2][i], 
               &g[j][3][i], &g[j][4][i], &g[j][5][i]);

    fclose(fp);

  }
 
  /*Linear interpolation of parameters*/
  if(((wavel<wlmin)||(wavel>wlmax))&&((wavel<wlp[0])||(wavel>wlp[11]+0.5)))
  {
    if(!flagwl){
      fprintf(stderr,"Error, wavelength %f micron out of yang parametrization range\n",
	      wavel);
      flagwl++;
    }
    return WLOUT;
    /*
    if(wavel<wlmin)
      wavel=wlmin;
    else
      wavel=wlmax;
      */
  }

  if(wavel<wlp[0]){
    ini=0;
    fin=index-1;
    med=(fin+ini)/2;
    
    while((med!=ini)&&(med!=fin))
      {
	if(wavel<=wl[med])
	  fin=med;
	else
	  ini=med;
	med=(fin+ini)/2;
      }
    
    auxd=1.0/reff;
    auxd1=ext[nhabit][0][ini]+ext[nhabit][1][ini]*auxd+
      ext[nhabit][2][ini]*auxd*auxd+ext[nhabit][3][ini]*auxd*auxd*auxd;  
    auxd2=ext[nhabit][0][fin]+ext[nhabit][1][fin]*auxd+
      ext[nhabit][2][fin]*auxd*auxd+ext[nhabit][3][fin]*auxd*auxd*auxd;
    *exti=auxd1+(wavel-wl[ini])*(auxd2-auxd1)/(wl[fin]-wl[ini]);

    auxd=reff;
    auxd1=ssa[nhabit][0][ini] + ssa[nhabit][1][ini]*auxd + ssa[nhabit][2][ini]*(auxd*auxd) + ssa[nhabit][3][ini]*(auxd*auxd*auxd);  
    auxd2=ssa[nhabit][0][fin] + ssa[nhabit][1][fin]*auxd + ssa[nhabit][2][fin]*auxd*auxd + ssa[nhabit][3][fin]*auxd*auxd*auxd;
    *ssai=auxd1+(wavel-wl[ini])*(auxd2-auxd1)/(wl[fin]-wl[ini]);
    
    auxd=reff;
    auxd1=g[nhabit][0][ini]+g[nhabit][1][ini]*auxd+
      g[nhabit][2][ini]*(auxd*auxd)+g[nhabit][3][ini]*(auxd*auxd*auxd);  
    auxd2=g[nhabit][0][fin]+g[nhabit][1][fin]*auxd+
      g[nhabit][2][fin]*auxd*auxd+g[nhabit][3][fin]*auxd*auxd*auxd;
    *gi=auxd1+(wavel-wl[ini])*(auxd2-auxd1)/(wl[fin]-wl[ini]);
    
    auxd1=g[nhabit][4][ini];
    auxd2=g[nhabit][4][fin];
    *g2i=auxd1+(wavel-wl[ini])*(auxd2-auxd1)/(wl[fin]-wl[ini]);
    
    auxd1=g[nhabit][5][ini];
    auxd2=g[nhabit][5][fin];
    *fi=auxd1+(wavel-wl[ini])*(auxd2-auxd1)/(wl[fin]-wl[ini]);
  }
  else{
    ini=0;
    fin=11;
    med=(fin+ini)/2;

    wavel= wavel > 13.0 ? 13.0 : wavel;
    
    while((med!=ini)&&(med!=fin))
      {
	if(wavel<=wlp[med])
	  fin=med;
	else
	  ini=med;
	med=(fin+ini)/2;
      }    

    auxd=1.0/(2.0*reff);
    auxd1=(3.0*1000.0/(0.9167*4.0*reff))*(qep[2][ini]*auxd*auxd+qep[1][ini]*auxd+qep[0][ini]);  
    auxd2=(3.0*1000.0/(0.9167*4.0*reff))*(qep[2][fin]*auxd*auxd+qep[1][fin]*auxd+qep[0][fin]);
    *exti=auxd1+(wavel-wlp[ini])*(auxd2-auxd1)/(wlp[fin]-wlp[ini]);

    auxd=1.0/(2.0*reff);
    auxd1=(ssp[2][ini]*auxd*auxd+ssp[1][ini]*auxd+ssp[0][ini]);  
    auxd2=(ssp[2][fin]*auxd*auxd+ssp[1][fin]*auxd+ssp[0][fin]);
    *ssai=auxd1+(wavel-wlp[ini])*(auxd2-auxd1)/(wlp[fin]-wlp[ini]);
    
    auxd=2.0*reff;
    auxd1=gp[0][ini]*pow(auxd,gp[1][ini]); 
    auxd2=gp[0][fin]*pow(auxd,gp[1][fin]);
    *gi=auxd1+(wavel-wlp[ini])*(auxd2-auxd1)/(wlp[fin]-wlp[ini]);
  
    *g2i=0.5;  /*Arbitrary*/
    *fi=1.0;   /*Only forward*/
  }

  return 0;
}
/*agf*/


