/*--------------------------------------------------------------------
 * $Id: cldprp.c,v 1.8 2001/12/18 15:06:50 pa2h Exp $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2001 by Arve Kylling and Bernhard Mayer.
 * 
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/

/*  @18c@                

@code{cldprp} calculates wavelength-dependent cloud properties 
using one of several parameterizations.

The different options to @code{cldprp} are displayed when executing:

@example
cldprp -h
@end example

    @c18@ */


#include <ascii.h>
#include <numeric.h>
#include <getopt.h>
#include "../src_f/f77-uscore.h"


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#if HAVE_YANG56
#include "../src_c/yang56.h"
#endif



#define PROGRAM "CLDPRP"
#define VERSION "0.80a"
#define DATE    "November 23, 2001"


#define HU   1
#define YANG 2


static void print_usage (char *name)
{
  fprintf (stderr, "usage:   %s  <input file>\n", name);
  fprintf (stderr, "%s calculates optical properties of water clouds\n", PROGRAM);
  fprintf (stderr,"according to Hu and Stamnes [1993].\n\n");
  fprintf (stderr, "Required arguments:\n");
  fprintf (stderr, "  The input file is a three-column file with wavelength [nm],\n");
  fprintf (stderr, "  liquid water content [g/m3], and effective droplet radius [um];\n");
  fprintf (stderr, "  for the Yang et al parameterization, a 4th column may be specified\n");
  fprintf (stderr, "  with the particle habit.\n\n");
  fprintf (stderr, "Optional arguments:\n");
  fprintf (stderr, "  -h            Show this help page\n");
  fprintf (stderr, "  -y            Use Yang et al. ice cloud parameterization\n");
  fprintf (stderr, "                (default: Hu and Stamnes)\n\n");
  fprintf (stderr, "The output is six-columns: wavelength [nm],\n");
  fprintf (stderr, "liquid water content [g/m3], effective droplet radius [um],\n");
  fprintf (stderr, "the extinction coefficient [1/km], the asymmetry parameter,\n");
  fprintf (stderr, "and the single scattering albedo. For the Yang et al. parameterization,\n");
  fprintf (stderr, "g1, g2, and f are printed as well.\n");
  
}




/*************************************************************/
/* parse command line options                                */
/*************************************************************/

static int get_options (int argc, char **argv,
			char *programname,
			char *infilename,
			int *properties)
{
  int c=0;

  /* reset parameters */
  strcpy (infilename, "");
  *properties=HU;

  /* save name of program */
  strncpy (programname, argv[0], FILENAME_MAX);
  
  

  while ((c=getopt (argc, argv, "hy")) != EOF)  {
    switch(c)  {
    case 'y':  /* use Yang et al */
      *properties = YANG;
      break;
    case 'h':  /* help */
    case '?':
    default:
      print_usage (programname);
      return (-1);
    }
  }
      
  /* check number of remaining command line arguments */
  if (argc - optind != 1)  {
    print_usage (argv[0]);
    return -1;
  }

  strcpy (infilename, argv[optind]);

  return 0;  /* if o.k. */
}





int main (int argc, char **argv)
{
  int i=0, status=0;
  char path[FILENAME_MAX]="";
  char yangpath[FILENAME_MAX]="";
  int npath;

  float *wvl=NULL, *lwc=NULL, *reff=NULL, *habit=NULL;
  float maxreff=0;
  int hab=0;

  int n=0;

  int properties=0;


  void  F77_FCN (wcloud) (float* lambda_t, int* newsiz, int* nlyr,
			  char* filepath, int* nstring, float* wccon, float* wceffr, 
			  float* tmp_wc_dtau, float* tmp_wc_gg, float* tmp_wc_ssa, float* zd,
			  int *wclyr);


  char programname[FILENAME_MAX], infilename[FILENAME_MAX];

  int newsiz=1, nlyr=1, wclayer=1, nohabit=0;
  float tau=0, gg=0, ssa=0, g1=0, g2=0, f=0;

  float zd[2]    = {1.0, 0.0};
  float effr[2]  = {0.0, 0.0};
  float wccon[2] = {0.0, 0.0};

  /* get command line options */
  status = get_options (argc, argv,
			programname, infilename, &properties);
  
  if (status!=0)
    return status;
  

  fprintf (stderr, " ... reading data from %s\n", infilename);


  /* read wavelength file */
  status = read_4c_file_float (infilename, &wvl, &lwc, &reff, &habit, &n);

  if (status!=0) {
    nohabit=1;

    if (properties==YANG)
      fprintf (stderr, " ... warning, no habit defined, assuming 0\n");

    /* try to read 3 columns */
    status = read_3c_file_float (infilename, &wvl, &lwc, &reff, &n);
    
    if (status!=0) {
      fprintf (stderr, "Error %d reading %s\n", status, infilename);
      return status;
    }
  }





  strcpy (path, "../data/");
  strcpy (yangpath, "../data/ic/yang56/");


  npath = strlen(path);

  for (i=0; i<n; i++) {
    effr[1]  = reff[i];
    wccon[1] = lwc[i];

    if (!nohabit)
      hab = (int) (habit[i]+0.5);
    else 
      hab = 0;

    switch(properties) {

    case HU:
      F77_FCN (wcloud) (&(wvl[i]), &newsiz, &nlyr,
			path, &npath, wccon, effr, 
			&tau, &gg, &ssa, 
			zd, &wclayer);
      

      
      fprintf (stdout, "%g %g %g %g %g %g\n", wvl[i], lwc[i], reff[i], tau, gg, ssa);
      
      break;
      
    case YANG:

#if HAVE_YANG56

      switch (hab)  {
      case IC_HABIT_SOLIDCOLUMN:   
	maxreff=MAX_REFF_SC; 
	break;
      case IC_HABIT_HOLLOWCOLUMN:  
	maxreff=MAX_REFF_HC; 
	break;
      case IC_HABIT_AGGREGATE:     
	maxreff=MAX_REFF_RA; 
	break;
      case IC_HABIT_ROSETTE4:      
	maxreff=MAX_REFF_R4; 
	break;
      case IC_HABIT_ROSETTE6:      
	maxreff=MAX_REFF_R6; 
	break;
      case IC_HABIT_PLATE:         
	maxreff=MAX_REFF_PL; 
	break;
      default:
	fprintf (stderr, "Error, unknown habit %d\n", hab);
	return -1;
      }
      
      if (reff[i]>maxreff) {
	fprintf (stderr, "*** Warning, effective radius %f larger than maximum\n", reff[i]);
	fprintf (stderr, "*** allowed value %f for this habit\n", maxreff);
	
	tau = NAN;
	ssa = NAN;
	g1  = NAN;
	g2  = NAN;
	f   = NAN;
      }
      else {
	status = yang56 (wvl[i]/1000.0, reff[i], hab, yangpath, 
			 &tau, &ssa, &g1, &g2, &f);
	
	if (status!=0) {
	  fprintf (stderr, "Error %d returned by yang56()\n", status);
	  return status;
	}
      }


      fprintf (stdout, "%g %g %g %g %g %g %g %g %g\n", wvl[i], lwc[i], reff[i], 
	       tau*lwc[i], g1*f+(1.0-f)*g2, ssa,
	       g1, g2, f);
#else
      fprintf (stderr, " ***********************************************************************\n");
      fprintf (stderr, " * You have built uvspec without Yang et al support and hence cannot   *\n");
      fprintf (stderr, " * use 'ic_properties yang'.                                           *\n");
      fprintf (stderr, " ***********************************************************************\n");
      return -1;
#endif

      break;

    default:
      fprintf (stderr, "Error, unknown cloud properties %d", properties);
      return status;
    }
  }


  return 0;
}
