/*--------------------------------------------------------------------
 * $Id: mie.c,v 1.6 2001/12/17 09:22:34 pa2h Exp $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2001 by Arve Kylling and Bernhard Mayer.
 * 
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../src_c/ascii.h"
#include "../src_c/getopt.h"
#include "./mie.h"



int mie (input_struct input, output_struct *output)
{
  int im=0, ip=0, ir=0, is=0, iv=0, status=0;
  
  double *ri_real=NULL, *ri_cplx=NULL;

  float temperature = 300.0;  /* The temperature is only used for wavelengths > 167 um */
  double wavelength=0;

  double beta=0, omega=0, g=0;

  double *x_size=NULL, *y_size=NULL;
  int n_size=0;

  mie_complex ref = {0.0, 0.0};



  /* read refractive index */
  if (strlen (input.ri_filename)>0) {
    status = read_3c_file (input.ri_filename, 
			   &output->wl.lambda, &ri_real, &ri_cplx, &output->wl.nlambda);
    
    if (status != 0) {
      fprintf (stderr, "error %d reading %s\n", status, input.ri_filename);
      return status;
    }

    fprintf (stderr, " ... read %d data points from %s\n", 
	     output->wl.nlambda, input.ri_filename);


    /* check if data points are sorted in ascending order */
    for (ir=0; ir<output->wl.nlambda-1; ir++)
      if (output->wl.lambda[ir+1] < output->wl.lambda[ir]) {
	fprintf (stderr, "error: %s not sorted in ascending order\n", 
		 input.ri_filename);
	return -1;
      }

    /* special convention here */
    output->wl.nlambda--;
  }
  else {
    output->wl.nlambda = (int) (input.wl.wl_end-input.wl.wl_start) / input.wl.wl_step;
    output->wl.lambda = (double *) calloc (output->wl.nlambda+1, sizeof(double));
    for (iv=0; iv<=output->wl.nlambda; iv++)
      output->wl.lambda[iv] = input.wl.wl_start + input.wl.wl_step*iv;
  }


  output->mie.pmom = calloc (4, sizeof(float *));
  for (ip=0; ip<4; ip++)
    output->mie.pmom[ip] = calloc (input.mie.nmom+1, sizeof(float));
  

  /* read size distribution */
  if (strlen (input.sd_filename)>0) {
    status = read_2c_file (input.sd_filename, &x_size, &y_size, &n_size);
    
    if (status != 0) {
      fprintf (stderr, "error %d reading %s\n", status, input.sd_filename);
      return status;
    }

    fprintf (stderr, " ... read %d data points from %s\n", 
	     n_size, input.sd_filename);


    /* check if data points are sorted in ascending order */
    for (is=0; is<n_size-1; is++)
      if (x_size[is+1]<x_size[is]) {
	fprintf (stderr, "error: %s not sorted in ascending order\n", 
		 input.sd_filename);
	return -1;
      }
    
    if (x_size[0]==0) {
      x_size[0] = 0.1 * x_size[1];
      fprintf (stderr, " ... cannot handle r=0; setting first data point to r=%g\n",
	       x_size[0]);
    }
  }



  for (iv=0; iv<=output->wl.nlambda; iv++) {

    wavelength = (output->wl.lambda[iv]) * 1.e-03;  /* Convert from nm to um */

    if (strlen (input.ri_filename)>0) {
      input.crefin.re = ri_real[iv];
      input.crefin.im = ri_cplx[iv];
    }

    if (strlen (input.sd_filename)>0) {

      status = mie_calc_sizedist (input.mie, &(output->mie), 
				  input.program, input.medium, input.crefin,
				  wavelength, temperature, x_size, y_size, n_size,
				  &beta, &omega, &g, &ref);

      if (status!=0) {
	fprintf (stderr, "error %d returned by mie_calc_sizedist()\n", status);
	return status;
      }


      fprintf(stdout, "%f %e %e %e %e %e %e", 
	      output->wl.lambda[iv], ref.re, ref.im,
	      beta, omega, g, -1.0);
      
      /* print normalized moments */
      if (input.mie.nmom>0)
	for (im=0; im<=input.mie.nmom; im++)
	  fprintf (stdout, " %e", output->mie.pmom[0][im]/output->mie.pmom[0][0]);

      fprintf (stdout, "\n");
      fflush (stdout);
    }
    else {

      status = mie_calc (input.mie, &(output->mie), 
			 input.program, input.medium, input.crefin,
			 wavelength, input.r_mean, temperature, &ref);

      if (status!=0) {
	fprintf (stderr, "error %d returned by mie_calc()\n", status);
	return status;
      }
      
      fprintf(stdout, "%f %e %e %f %e %f %e", 
	      output->wl.lambda[iv], ref.re, ref.im, output->mie.qext, 
	      output->mie.qsca/output->mie.qext, 
	      output->mie.gsca, output->mie.spike);

      /* print normalized moments */
      if (input.mie.nmom>0)
	for (im=0; im<=input.mie.nmom; im++)
	  fprintf (stdout, " %e", output->mie.pmom[0][im]/output->mie.pmom[0][0]);

      fprintf (stdout, "\n");
      fflush (stdout);
    }

    /* for debugging: print cumulative probability distribution      */
    /* to 'cumulative.dat':                                          */
    /* cumulative_probability (output->mie.pmom[0], input.mie.nmom); */

    /* for debugging: print phase function to 'phase.dat':           */
    /* phase_function (output->mie.pmom[0], input.mie.nmom);         */
  }
  

  return 0;
}

